﻿/* Wrapper for media library database
   Created by Zoominla, 02-16-2009         
*/

if(typeof(MEDIA_DATABASE_INCLUDE) != "boolean") {
	MEDIA_DATABASE_INCLUDE = true;
    if (typeof(Cc) == "undefined")
        var Cc = Components.classes;
    if (typeof(Ci) == "undefined")
        var Ci = Components.interfaces;
    function MediaDb(dbFile)
    {
        var isDbExist = dbFile.exists();
        var storageService = Cc["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService);
        // open the database connection and create table
        this.conn = storageService.openDatabase(dbFile);
		this.mediaTableName = 'medialibTable';
		this.plTableName = 'playlistTable';
		this.initDone = false;
		var predefineListName = ['HomeVideo', 'Movie', 'Episode'];
		this.customListStartId = predefineListName.length + 1;
		
		if(!isDbExist) {	// if db file is not exist, create tables
			var mediaFields = ['Title STRING', 'Duration STRING', 'Size STRING',
							   'Genre STRING', 'Artist STRING','Album STRING',
							   'LastPlayDate STRING', 'Location STRING', 'AddDate STRING',
							   'Year STRING','Comment STRING', 'Width INTEGER', 'Height INTEGER','ListId INTEGER',
							   'OnMobile INTEGER','PlayCount INTEGER', 'Type INTEGER','Rate INTEGER'];
			var plFields = ['ListId INTEGER PRIMARY KEY', 'ListName STRING', 'FilterName STRING',
					'DisplayFields STRING', 'Current INTEGER', 'ReverseOrder INTEGER'];
			
			var mediaSql = 'CREATE TABLE IF NOT EXISTS '+ this.mediaTableName + ' (' +
							mediaFields.join(',') + ')';
			var plSql = 'CREATE TABLE IF NOT EXISTS '+ this.plTableName + ' (' +
							plFields.join(',') + ')';
			this.conn.executeSimpleSQL(mediaSql);
			this.conn.executeSimpleSQL(plSql);
			
			// Add predefined data rows to playlist table (playlists)
			for each (var listName in predefineListName) {
				this.addPlaylist(listName);
			}
		}
		
		// Observer pattern related properties
		this.observers = [];
		this.updateMsg = null;
		
		// Init query functions
		this.mediaFieldsUsed = ['Title','Duration','Size','Genre','Artist','Album','Location'];
		this.listFieldsUsed = ['ListId', 'ListName'];
		var mcols = this.mediaFieldsUsed.join(',');
		var lcols = this.listFieldsUsed.join(',');
		this.queryFunction( {
			getAllMedias: 'SELECT '+mcols+' FROM '+this.mediaTableName,
			getGenres: 'SELECT DISTINCT Genre FROM '+this.mediaTableName+' WHERE Type = %i1',
			getArtists: 'SELECT DISTINCT Artist FROM '+this.mediaTableName+' WHERE Type = %i1',
			getAlbums: 'SELECT DISTINCT Album FROM '+this.mediaTableName+' WHERE Type = %i1',
			getYears: 'SELECT DISTINCT Year FROM '+this.mediaTableName+' WHERE Type = %i1',
			getAddDates: 'SELECT DISTINCT AddDate FROM '+this.mediaTableName+' WHERE Type = %i1',
			getCustomLists: 'SELECT '+lcols+' FROM ' + this.plTableName + ' WHERE ListId >= %i1',
			getMediaByListId: 'SELECT '+mcols+' FROM '+this.mediaTableName+' WHERE ListId = %i1',
			getAllLists: 'SELECT '+lcols+' FROM ' + this.plTableName
		});
		
		this.initDone = true;
    };

	MediaDb.LISTENER_FLAGS_ITEMADDED = 0;
 	MediaDb.LISTENER_FLAGS_BEFOREITEMREMOVED = 1;
 	MediaDb.LISTENER_FLAGS_AFTERITEMREMOVED = 2;
 	MediaDb.LISTENER_FLAGS_ITEMUPDATED = 3;
 	MediaDb.LISTENER_FLAGS_LISTCLEARED = 4;
	MediaDb.LISTENER_FLAGS_LISTADDED = 5;
	MediaDb.LISTENER_FLAGS_LISTREMOVED = 6;
 	MediaDb.LISTENER_FLAGS_ITEMMOVED = 7;
 	MediaDb.LISTENER_FLAGS_ALL = 8;
	
	MediaDb.FIELD_TYPE_INTEGER = 0;
	MediaDb.FIELD_TYPE_STRING = 1;
	
	MediaDb.prototype.initialize = function() {
		
	};
	
	//---------------- Database operation ---------------------------------
	// Add data row to mediaTable
	// @Param: rowDataObj is a object. ex: {Title:'One more time', Duration:'04:30'...}
	//         Or it's an array of such type of object
	MediaDb.prototype.addMedia = function(rowDataObj) {
		if(!rowDataObj) return;
		if(rowDataObj instanceof Array) {	// Add some media file
			for(var i=0; i<rowDataObj.length; ++i) {
				this.addOneMedia(rowDataObj[i]);
			}
		} else {							// Add one media file
			this.addOneMedia(rowDataObj);
		}
		
		// update listeners
		if(this.initDone) {
			this.setState(MediaDb.LISTENER_FLAGS_ITEMADDED);
		}
	};
	
	// Add data row to playlistTable
	// @Param: listName, a string type
	MediaDb.prototype.addPlaylist = function(listName) {
		if(!listName) return;
		var lstname = listName || '';
		var fields = [0, lstname, '', '', 0, 0];
		var insertSql = 'INSERT INTO ' + this.plTableName + ' VALUES (?1, ?2, ?3, ?4, ?5,?6)';
		//dumpMsg(insertSql);
		var statement = null;
		try {
			statement = this.conn.createStatement(insertSql);
			statement.bindNullParameter(0);
			for(var i=1; i<fields.length; ++i) {
				if(i<fields.length-2) {
					statement.bindUTF8StringParameter(i, fields[i]);
				} else {
					statement.bindInt32Parameter(i, fields[i]);
				}
			}
			statement.execute();
		} catch (e) {
			//dumpMsg(e);
			if(statement) statement.reset();
		}
		
		// update listeners
		if(this.initDone) {
			this.setState(MediaDb.LISTENER_FLAGS_LISTADDED);
		}
		//dumpMsg('Add list: ' + listName);
	};
	
	MediaDb.prototype.getFieldType = function(fieldName) {
		switch(fieldName) {
			case 'Width':
			case 'Height':
			case 'ListId':
			case 'OnMobile':
			case 'PlayCount':
			case 'Type':
			case 'Rate':
				return MediaDb.FIELD_TYPE_INTEGER;
			default:
				return MediaDb.FIELD_TYPE_STRING;
		}
	};
	
	// Get fields used in SELECT clause in mediaTable
	// @Return: A string array
	MediaDb.prototype.getMediaTableUsedFields = function() {
		return this.mediaFieldsUsed;
	};
	
	// Get fields used in SELECT clause in playlistTable
	// @Return: A string array
	MediaDb.prototype.getListTableUsedFields = function() {
		return this.listFieldsUsed;
	};
	
	// Get the start id of custom lists
	MediaDb.prototype.getCustomListStartId = function() {
		return this.customListStartId;
	};
	
	// Get the name array of custom lists
	MediaDb.prototype.getCustomLists = function() {
		return this.queries['getCustomLists'](this.customListStartId);
	};
	
	// Get names array of predefined lists
	// @Return: A string array
	MediaDb.prototype.getPredefineListNames = function() {
		return this.predefineListName;
	};
	
	//------------------ Query functions --------------------------
	
	//Query via a sql string
	//@Params: sqlStr(a sql string), other parameters are optional 
	MediaDb.prototype.queryBySQL = function(sqlStr/*,...*/) {
		if(!sqlStr) return null;
		this.queryFunction("tempFunc", sqlStr);
		// The paramter count usally less than 4
		switch(arguments.length-1) {
			case 0:
				return this.queries['tempFunc']();
			case 1:
				return this.queries['tempFunc'](arguments[1]);
			case 2:
				return this.queries['tempFunc'](arguments[1],arguments[2]);
			case 3:
				return this.queries['tempFunc'](arguments[1], arguments[2], arguments[3]);
			case 4:
				return this.queries['tempFunc'](arguments[1], arguments[2], arguments[3], arguments[4]);
			case 5:
				return this.queries['tempFunc'](arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
		}
		return null;
	};
	
	MediaDb.prototype.getAllMedia = function() {
		return this.queries['getAllMedias']();
	};
	MediaDb.prototype.getAllLists = function(typeId) {
		return this.queries['getAllLists'](typeId);
	};
	MediaDb.prototype.getMediaByListId = function(listIndex) {
		return this.queries['getMediaByListId'](listIndex);
	};
	MediaDb.prototype.getGenres = function(typeId)
	{
		return this.queries['getGenres'](typeId);
	};
	MediaDb.prototype.getArtists = function(typeId)
	{
		return this.queries['getArtists'](typeId);
	};
	MediaDb.prototype.getAlbums = function(typeId)
	{
		return this.queries['getAlbums'](typeId);
	};
	MediaDb.prototype.getAddDates = function(typeId)
	{
		return this.queries['getAddDates'](typeId);
	};
	MediaDb.prototype.getYears = function(typeId)
	{
		return this.queries['getYears'](typeId);
	};
	
	//--------------- Subject related Methods ---------------------
	MediaDb.prototype.addObserver = function(observerObj) {
		this.observers.push(observerObj);
	};
	MediaDb.prototype.removeObserver = function(observerObj) {
		var i = 0;
		for(; i<this.observers.length; ++i) {
			if(this.observers[i] == observerObj) break;
		}
		this.observers.splice(i, 1);
	};
	MediaDb.prototype.notify = function() {
		for(var i=0; i<this.observers.length; ++i) {
			this.observers[i].update(this.updateMsg);
		}
	};
	
	MediaDb.prototype.setState = function(eventType) {
		this.updateMsg = eventType;
		this.notify();
	};
	
	MediaDb.prototype.close = function() {
		if(this.conn) {
			this.conn.close();
		}
		this.conn = null;
	};
	
	//----------------- Helper functions ---------------------------------
	// Add one data row to mediaTable
	// @Param: rowDataObj is a object. ex: {Title:'One more time', Duration:'04:30'...}
	MediaDb.prototype.addOneMedia = function(rowDataObj) {
		if(!rowDataObj) return;
		var fields = [];
		
		fields.push(rowDataObj.Title, rowDataObj.Duration);
		fields.push(rowDataObj.Size, rowDataObj.Genre);
		fields.push(rowDataObj.Artist, rowDataObj.Album);
		fields.push(rowDataObj.LastPlayDate, rowDataObj.Location);
		fields.push(rowDataObj.AddDate, rowDataObj.Year, rowDataObj.Comment);
		// Integer type
		fields.push(rowDataObj.Width || 0, rowDataObj.Height || 0,rowDataObj.ListId || 0);
		fields.push(rowDataObj.OnMobile || 0, rowDataObj.PlayCount || 0, rowDataObj.Type || 0, rowDataObj.Rate || 0);
		
		var insertSql = 'INSERT INTO ' + this.mediaTableName + ' VALUES (' +
						'?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9, ?10, ?11, ?12, ' +
						'?13, ?14, ?15, ?16, ?17, ?18)';
		var statement = null;
		try {
			//dumpMsg(insertSql);
			statement = this.conn.createStatement(insertSql);
			for(var i=0; i<fields.length; ++i) {
				if(i<fields.length-7) {
					statement.bindUTF8StringParameter(i, fields[i]);
				} else {
					statement.bindInt32Parameter(i, fields[i]);
				}
			}
			statement.execute();
		} catch (e) {
			//dumpMsg(e);
			if(statement) statement.reset();
		}
	};
	
	
	var RE_SET_ORDER = 1,
    RE_MATCH_ORDER = 0; //constants for matchAll
    String.prototype.matchAll = function(re, set) {
        //Required for SQLite parameter matching
        if (!set) set = RE_MATCH_ORDER;
        //copy source string, initialize output array
        var ret = [],
        ts = this;
        while (re.test(ts)) {
            //match
            ret.push(ts.match(re));
            //remove match
            ts = ts.replace(re, '');
        }
        //If we have to swap orders, do so.
        if (set == RE_SET_ORDER) {
            var i, j, nRet = [];
            for (i = 0; i < ret.length; i++) {
                for (j = 0; j < ret[i].length; j++) {
                    if (i == 0) nRet[j] = [];
                    nRet[j].push(ret[i][j]);
                }
            }
            ret = nRet;
        }
        return ret;
    };
	//Cache for SQLite queries
	MediaDb.prototype.queries = {};
	// Create new query named functions to select data from database.
	MediaDb.prototype.queryFunction = function(name, query) {
        //If name is an object
        if (typeof name == 'object') {
            //initialize our return value
            var queryGroup = {};
            //for each function NVP
            for (var i in name) {
                //create a matching queryFunction
                queryGroup[i] = arguments.callee.apply(this, [i, name[i]]);
            }
            //return the query namespace
            return queryGroup;
        }

        //allow the internal function to access this SQLite instance
        var _this = this;
        //use _conn as a shortcut if necessary
        var _conn = this.conn;
        //Match all % parameters (for type matching)
        var params = query.matchAll(/%(i|n|s|f)(\d+)/i);
        //And replace with their real counterparts
        var realQuery = query.replace(/%(i|n|s|f)(\d+)/ig, '?$2');
        //dumpMsg(realQuery);
        //create the statement
        var statement = _conn.createStatement(realQuery);       
        //Create the query function, loaded with the variables available above.
        var exec = function() {
            var i, idx;
            //walk through the parameters
            for (i = 0; i < params.length; i++) {
                //get the param index for the parameter
                idx = statement.getParameterIndex('?' + params[i][2]);
                switch (params[i][1].toLowerCase()) {
                    //it's a int32 number.
                case 'i':
                    statement.bindInt32Parameter(idx, arguments[idx]);
                    break;
                case 'n':
                    //it's a int64 number.
                    statement.bindInt64Parameter(idx, arguments[idx]);
                    break;
                case 'f':
                    //it's a float.  Use doubles.
                    statement.bindDoubleParameter(idx, arguments[idx]);
                    break;
                default:
                    //Assume it's a UTF8 string; ASCII and blobs will handle this fine on the input side
                    statement.bindUTF8StringParameter(idx, arguments[idx]);
                }
            }
            //Some variables for the return
            var cols = statement.columnCount,
            col = 0,
            colTypes = [],
            colNames = [],
            ret = [],
            colName,
            rowData;
            //If we've requested columns
            if (cols > 0) {
                while (statement.executeStep()) {
                    //execute like we're expecting a return value.
                    rowData = {};
                    for (col = 0; col < cols; col++) {
                        if (!colNames[col]) {
                            //populate the colNames and colTypes arrays, as needed
                            colNames[col] = statement.getColumnName(col);
                            colTypes[col] = statement.getTypeOfIndex(col);
                        }
                        colName = colNames[col];
                        switch (colTypes[col]) {
                            //Depending on the column type (this really matters on output)
                            //fill the rowData object
                        case 0:
                            rowData[colName] = null;
                            break;
                        case 1:
                            rowData[colName] = statement.getInt64(col);
                            break;
                        case 2:
                            rowData[colName] = statement.getDouble(col);
                            break;
                        case 3:
                            rowData[colName] = statement.getUTF8String(col);
                            break;
                        case 4:
                            rowData[colName] = statement.getBlob(col);
                            break;
                        }
                    }
                    //And, once this row is done, append it to the array.
                    ret.push(rowData);
                }
            } else {
                //If there's no columns to grab, just run the query
                statement.execute();
            }
            //Either way, reset to stave off hobgoblins
            statement.reset();
            if (ret.length > 0) //We've got return values?  Good!  Send 'em up.
            return ret;
            //We don't?  Well, an insert will return the last insert ID
            //and a no-value select will return 0.
            return _conn.lastInsertRowID;
        };
        //Add exec to our set of precached queries, and return it.
        return this.queries[name] = exec;
    };
}