﻿/* Wrapper for media library database
   Created by Zoominla, 02-16-2009         
*/

if(typeof(MEDIA_MANAGER_INCLUDE) != "boolean") {
	MEDIA_MANAGER_INCLUDE = true;
	function MediaMan() {
		//Create navigate tree box and playlist tree box
		this.navTree = new CTreeBox($('mediaManager'));
		this.listTree = new CTreeBox($('mediaList'));
		
		// Find full path of category.xml
		var mediaDbFile = Io.getFileFrom(userDataPath);
		if(mediaDbFile) {
			mediaDbFile.append("playlists");
			if(mediaDbFile && !mediaDbFile.exists()) {
				mediaDbFile.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 420);
			}
			if(mediaDbFile) {
				mediaDbFile.append('mediadb.sqlite');
			}
		}
		this.mediaDb = new MediaDb(mediaDbFile);
		this.mediaDb.addObserver(this.listTree);	//Let playlist tree observe media database
		
		//Initialize navigate tree box and playlist tree box
		this.navTree.initialize(this.mediaDb);
		this.listTree.initialize(this.mediaDb);
		
		var navXmlFile = this._getNavXmlFile();
		if(!navXmlFile && !navXmlFile.exists()) {
			throw new Error("category.xml don't exist!");
		}
		this.navigator = new NavCatalog(navXmlFile);
		this.navigator.initialize(this.mediaDb, this.navTree, this.listTree);
		
		this.curPlaylist = null;
		this._mapEvents();
	};
	
	//----------------- Operation on medialib ---------------------------
	MediaMan.prototype.addMedia = function(rowDataObj) {
		this.mediaDb.addMedia(rowDataObj);
	};
	
	//Get Current file for playing in current playlist
	MediaMan.prototype.getCurrentFile = function() {
		if(this.curPlaylist) {
			var idx = this.listTree.curIndex;
			return this.curPlaylist[idx].Location;
		}
		return null;
	};
	
	//Get filename of the selected iterm in current playlist
	MediaMan.prototype.getSelectedFile = function() {
		var selIdx = this.listTree.getSelectedIndex();
		var listRowObj = this.listTree.tree.view.getObjectForIndex(selIdx);
		if(listRowObj) return listRowObj.Location;
		return null;
	};
	
	MediaMan.prototype.moveNext = function() {
		if(this.curPlaylist) {
			var idx = this.listTree.curIndex + 1;
			if(idx>0 && idx<this.curPlaylist.length) {
				this.listTree.curIndex = idx;
			}
		}
	};
	
	MediaMan.prototype.movePrevious = function() {
		if(this.curPlaylist) {
			var idx = this.listTree.curIndex - 1;
			if(idx>=0 && idx<this.curPlaylist.length) {
				this.listTree.curIndex = idx;
			}
		}
	};
	
	//----------------------- Events --------------------------------
	MediaMan.prototype.onNavigatorSelect = function(e) {
		this.navigator.onSelect(e);
	};
	
	MediaMan.prototype.onNavigatorConfirm = function(e) {
		this.navigator.onConfirm(e);
	};
	
	MediaMan.prototype.onNavigatorReturn = function(e) {
		this.navigator.onReturn(e);
	};
	
	MediaMan.prototype.onNavigatorkeyup = function(e) {
		//dumpMsg(e.keyCode);
		switch(e.keyCode) {
		case 8:		//BackSpace
			this.navigator.onReturn(e);
			break;
		}
		
	};
	
	MediaMan.prototype.onMediaListConfirm = function(e) {
		this.curPlaylist = this.listTree.execSQL();
		if(this.curPlaylist == null) return;
		this.listTree.setCurrentIndex();
		var curIdx = this.listTree.getCurrentIndex();
		startPlay(this.curPlaylist[curIdx].Location);
		playFromLibrary = true;		// play from medialibrary
	};
	
	MediaMan.prototype.onMediaListkeyup = function(e) {
		//dumpMsg(e.keyCode);
		switch(e.keyCode) {
		case 8:		//BackSpace
		
			break;
		}
		
	};
    //Get record from media library, each record is an object. (ex. {Title:'crysis', Duration:'0:20:35', ...})
	//@Params: filePath, A file path or an array
    //@return: A object or an object array
	MediaMan.prototype.getMediaObject = function(filePath) {
		if(!filePath) return null;
        if(filePath instanceof Array) {
            var objArr = [];
            for each (var fname in filePath) {
                var obj = this._getOneMediaObject(fname);
                if(obj) objArr.push(obj);
            }
			return objArr;
        } else if(typeof filePath == 'string') {
            return this._getOneMediaObject(filePath);
        }
        return null;
	};
    
	MediaMan.prototype.closeMediaDb = function() {
		if(this.mediaDb) {
			this.mediaDb.close();
		}
	}
    //--------------------- Inner functions -------------------------
    MediaMan.prototype._getOneMediaObject = function(filePath) {
        if(!mc.dumpMediaInfo) { 
            return null;
        }
        var resXml = mc.dumpMediaInfo(filePath, false);
        if(resXml == null)return null;
        var xmlDoc = new XMLDom();
        xmlDoc.loadXML(resXml);
        var rootNode = DomHelper.getChildNode(xmlDoc);
        var node = DomHelper.getChildNode(rootNode);
		var Title_STRING = DomHelper.getNodeValueByKey(node, "tags.title");
            if(!Title_STRING) Title_STRING = '';
        var Duration_STRING = DomHelper.getNodeValueByKey(node, "overall.duration"); 
            if(!Duration_STRING) Duration_STRING = '0';
            Duration_STRING = mp.getTimeString(parseInt(Duration_STRING) / 1000);
        var Size_STRING = DomHelper.getNodeValueByKey(node, "overall.size");
            if(!Size_STRING) Size_STRING = '0';
            Size_STRING = normalizeFileSize(parseInt(Size_STRING) / 1024);
        var Genre_STRING = DomHelper.getNodeValueByKey(node, "tags.genre"); 
            if(!Genre_STRING) Genre_STRING = '';
        var Artist_STRING = DomHelper.getNodeValueByKey(node, "tags.artist"); 
            if(!Artist_STRING) Artist_STRING = '';
        var Album_STRING = DomHelper.getNodeValueByKey(node, "tags.album"); 
            if(!Album_STRING) Album_STRING = '';
        var LastPlayDate_STRING  = '';        
        var Location_STRING = DomHelper.getNodeValueByKey(node, "overall.file");
            if(!Location_STRING) return null;
    
		var Width_INTEGER = DomHelper.getNodeValueByKey(node, "video.width");
			Width_INTEGER = parseInt(Width_INTEGER);
		var Height_INTEGER = DomHelper.getNodeValueByKey(node, "video.height");
			Height_INTEGER = parseInt(Height_INTEGER);
        
        var Type_INTEGER;
            var ext = getFileExt(Location_STRING);
            Type_INTEGER = getContentType(ext);
            /* video 1, audio 0, img 2, other 3*/            
        var AddDate_STRING = rndtoday.getFullYear() + '-' + (rndtoday.getMonth() +1) + '-' + rndtoday.getDate();
        var Year_STRING = DomHelper.getNodeValueByKey(node, "tags.year");
            if(!Year_STRING) Year_STRING = '';
        var Comment_STRING = DomHelper.getNodeValueByKey(node, "tags.comment");
            if(!Comment_STRING) Comment_STRING = '';
    	
        var mediaObj = {Title:Title_STRING, Duration:Duration_STRING, Size:Size_STRING, Genre:Genre_STRING,
        Artist:Artist_STRING, Album:Album_STRING, LastPlayDate:LastPlayDate_STRING, Location:Location_STRING,
        Type:Type_INTEGER, AddDate:AddDate_STRING, Year:Year_STRING, Comment:Comment_STRING, ListId:0,
        OnMobile:0, PlayCount:0, Rate:0, Width:Width_INTEGER, Height: Height_INTEGER};
        return mediaObj;
    };
	
	MediaMan.prototype._enumMediaFiles = function(dir, bRecursive, /*ret*/filesArr)
	{
		if(!dir) return;
		var topDir = Io.getFileFrom(dir);
		if(!topDir && !topDir.isDirectory()) return;
		var files = topDir.directoryEntries;
		while(files.hasMoreElements()) {
			var afile = files.getNext();
			afile.QueryInterface(Components.interfaces.nsIFile);
			if(afile && afile.isFile()) {
				filesArr.push(afile.path);	
			} else if(afile && afile.isDirectory() && bRecursive) {
				this._enumMediaFiles(afile, bRecursive, filesArr);
			}
		}
	}	

	MediaMan.prototype._mapEvents = function() {
		var _me = this;
		//navgateTree events
		//navgateTree.onselect = function(e){_me.onNavigatorSelect(e)};
		this.navTree.tree.onclick = function(e){_me.onNavigatorSelect(e)};
		this.navTree.tree.ondblclick = function(e){_me.onNavigatorConfirm(e)};
		this.navTree.tree.onkeyup = function(e){_me.onNavigatorkeyup(e)};
		
		//medialistTree events
		this.listTree.tree.ondblclick = function(e){_me.onMediaListConfirm(e)};
		this.listTree.tree.onkeyup = function(e){_me.onMediaListkeyup(e)};
		
		// Map context menu events
		
	}
	
	MediaMan.prototype._getNavXmlFile = function(filePath) {
		var navXmlFile = Io.getSpecialDir(null, 'AChrom');
		if(navXmlFile) {
			navXmlFile.append('content');
		}
		if(navXmlFile && navXmlFile.exists()) {
			navXmlFile.append('xulplayer');
		}
		if(navXmlFile && navXmlFile.exists()) {
			navXmlFile.append('medialib');
		}
		if(navXmlFile && navXmlFile.exists()) {
			navXmlFile.append('category.xml');
		}
		if(navXmlFile && navXmlFile.exists()) {
			return navXmlFile;
		}
		return null;
	};
}