/* Wrapper for walk through navigate categroy.xml
   Created by Zoominla, 02-22-2009         
*/

if(typeof(NAVIGATE_CATALOG_INCLUDE) != "boolean") {
	NAVIGATE_CATALOG_INCLUDE = true;
    
    function NavCatalog(catalogFile)
    {
		this.doc = DomHelper.LoadLocalXMLFile(catalogFile);
		if(!this.doc) {
			throw new error("Open category.xml failed when initializing NavCatalog");
		}
		this.root = DomHelper.getChildNode(this.doc);
		this.curNode = this.root;
		this.mediaDb = null;
		this.navTree = null;
		this.playlistTree = null;
    }
    
	NavCatalog.LISTENER_FLAGS_ONSELECT = 10;
	NavCatalog.LISTENER_FLAGS_ONCONFIRM = 11;
	NavCatalog.LISTENER_FLAGS_ONRETURN = 12;
	
    NavCatalog.prototype = {
		initialize: function(mediadb, navTree, playlistTree) {
			this.mediaDb = mediadb;
			this.navTree = navTree;
			this.playlistTree = playlistTree;
			if(!this.mediaDb || !this.navTree || !this.playlistTree) {
				throw new Error("NavCatalog init failed!");
			}

			// Complete the doc object, get data from database and append to it
			this._updateMusicNode();
			this._updatePictureNode();
			this._updateCustomListNode();
			// Set current node to music node
			this.curNode =  this.root;
			this.onConfirm();
			this.setState(NavCatalog.LISTENER_FLAGS_ONSELECT);
			//dumpMsg(DomHelper.GetXmlContent(this.root));
		},
		
		goRoot : function() {
			this.curNode = this.root;
		},
		
		goParent : function() {
			var parentNode = this.curNode.parentNode;
			if(parentNode) {
				this.curNode = parentNode;
			}
		},
		
		getCurrentNode : function() {
			return this.curNode;
		},
		
		onSelect: function(e) {
			var parentnode = this.curNode.parentNode;
			if(parentnode && parentnode != this.doc) {
				this.navTree.setCurrentIndex();
				var curIndex = this.navTree.getCurrentIndex();
				//dumpMsg(curIndex);
				var curnode = DomHelper.getChildByIndex(parentnode,curIndex);
				//dumpMsg(curnode);
				if(curnode) {
					this.curNode = curnode;
				}
			}
			
			this.setState(NavCatalog.LISTENER_FLAGS_ONSELECT);
		},
		
		onConfirm: function(e) {
			var child = DomHelper.getChildNode(this.curNode);
			if(child) {
				this.setState(NavCatalog.LISTENER_FLAGS_ONCONFIRM);
				this.curNode = child;
			}
		},
		
		onReturn: function(e) {
			if(this.curNode && this.curNode.parentNode == this.root) {
				return;
			}
			this.goParent();
			var navNamesArr = [];
			var childNode = DomHelper.getChildNode(this.curNode.parentNode);
			for(;childNode; childNode=DomHelper.getNextNode(childNode)) {
				var text = childNode.getAttribute('text');
				if(text != null) {
					navNamesArr.push({'manCol':text});
				}
			}
			this.navTree.setNavListNamesArr(navNamesArr);
			this.setState(NavCatalog.LISTENER_FLAGS_ONRETURN);
		},
		
		// Get all the children's name of the current node
		getChildNamesArray : function() {
			var namesArr = new Array();
			var childNode = DomHelper.getChildNode(this.curNode);
			while(childNode) {
				var aname = childNode.getAttribute('text');
				namesArr.push(aname);
				childNode = DomHelper.getNextNode(childNode);
			}
			return namesArr;
		},
		
		//--------------------- Subject related functions --------------------
		
		notify : function() {
			switch(this.updateMsg) {
				case NavCatalog.LISTENER_FLAGS_ONSELECT:
					if(this._preparePlaylistQuery()) {
						this.playlistTree.update(this.updateMsg);
					}
					break;
				case NavCatalog.LISTENER_FLAGS_ONCONFIRM:
					if(this._prepareNavigatorListNames()){
						this.navTree.update(this.updateMsg);
					}
					break;
				case NavCatalog.LISTENER_FLAGS_ONRETURN:
					this.navTree.update(this.updateMsg);
					break;
			}
		},
	
		setState : function(eventType) {
			this.updateMsg = eventType;
			this.notify();
		},
		
		//-------------------- Inner functions -------------------------------
		//Prepare sql string and parameters for palylist tree box
		_preparePlaylistQuery: function() {
			var curKey = this.curNode.getAttribute('key');
			var curVal = this.curNode.getAttribute('value');
			var curOp = this.curNode.getAttribute('op');
			if(curVal == null) return false;
			if(curOp == null) curOp = '=';
			var fieldsUsed = this.mediaDb.getMediaTableUsedFields();
			var selector = '*';
			if(fieldsUsed && fieldsUsed.length > 0) {
				selector = fieldsUsed.join(',');
			}
			
			// Assembly sql query string (plus parent node's condition)
			var paramCount = 1;
			var paramType = 's';
			if(this.mediaDb.getFieldType(curKey) == MediaDb.FIELD_TYPE_INTEGER) {
				paramType = 'i';
			}
			var sqlStr = 'SELECT ' + selector + ' FROM '+this.mediaDb.mediaTableName+
						 ' WHERE '+curKey +curOp+' %'+paramType+paramCount;
			var paramsArr = [curVal];
			var parentnode = this.curNode.parentNode;
			while(parentnode && parentnode != this.root) {
				var pKey = parentnode.getAttribute('key');
				var pVal = parentnode.getAttribute('value');
				var pOp = parentnode.getAttribute('op');
				parentnode = parentnode.parentNode;
				if(pVal == null) continue;
				if(pOp == null) pOp = '=';
				var pParamType = 's';
				if(this.mediaDb.getFieldType(pKey) == MediaDb.FIELD_TYPE_INTEGER) {
					pParamType = 'i';
				}
				paramCount++;
				sqlStr += ' AND '+pKey +pOp+' %'+pParamType+paramCount;
				paramsArr.push(pVal);
			}
			this.playlistTree.setCurrentSQL(sqlStr);
			this.playlistTree.setSQLParamsArr(paramsArr);
			return true;
		},
		
		//Prepare item names for navigate tree box to display
		_prepareNavigatorListNames: function() {
			var childNode = DomHelper.getChildNode(this.curNode);
			if(childNode == null) {
				return false;
			}
			
			if(this.curNode != this.root) {		// Update doc tree
				var curKey = this.curNode.getAttribute('key');
				var curVal = this.curNode.getAttribute('value');
				if(curKey == 'Type' && curVal == '0') {
					this._updateMusicNode();
				} else if(curKey == 'Type' && curVal == '2') {
					this._updatePictureNode();
				} else if(curKey == 'ListId' && parseInt(curVal) == this.mediaDb.getCustomListStartId()) {
					this._updateCustomListNode();
				}
			}
			
			var navNamesArr = [];
			for(;childNode; childNode=DomHelper.getNextNode(childNode)) {
				var text = childNode.getAttribute('text');
				if(text != null) {
					navNamesArr.push({'manCol':text});
				}
			}
			this.navTree.setNavListNamesArr(navNamesArr);
			return true;
		},
		
		_updateMusicNode : function() {
			var topClassNode = DomHelper.getChildNode(this.root);
			// Loop through first class node
			for(;topClassNode; topClassNode=DomHelper.getNextNode(topClassNode)) {
				var parentKey = topClassNode.getAttribute('key');
				var parentVal = topClassNode.getAttribute('value');
				if(parentVal == '0') {  // music node
					var childNode = DomHelper.getChildNode(topClassNode);
					if(childNode) {		// TopClassNode has child node
						for(;childNode; childNode=DomHelper.getNextNode(childNode)) {
							var curKey = childNode.getAttribute('key');
							var curVal = childNode.getAttribute('value');
							if(curVal == null) {	// If in music category and current node's value is null
								// If it already has children, romove them
								if(DomHelper.getChildNode(childNode) != null) {
									DomHelper.removeChildren(childNode);
								}
								var objs = this.mediaDb.queries['get'+curKey+'s'](parentVal);
								//dumpMsg(objs);
								for each (var obj in objs) {
									var text = obj[curKey];
									var val = text;
									if(text == '') {
										text = getLocalString("medialib.media.otherType");
									}
									if(text) this._insertNode(this.doc, childNode, text, curKey, val);
								}
							}
						}
					}
				}
			}
		},
		
		_updatePictureNode : function() {
			var topClassNode = DomHelper.getChildNode(this.root);
			// Loop through first class node
			for(;topClassNode; topClassNode=DomHelper.getNextNode(topClassNode)) {
				var parentKey = topClassNode.getAttribute('key');
				var parentVal = topClassNode.getAttribute('value');
				if(parentVal == '2') {  // picture node
					// If it already has children, romove them
					if(DomHelper.getChildNode(topClassNode) != null) {
						DomHelper.removeChildren(topClassNode);
					}
					var pictureDates = this.mediaDb.getAddDates(parentVal);
					for each (var obj in pictureDates) {
						var text = obj['AddDate'];
						var val = text;
						if(text == '') {
							text = getLocalString("medialib.media.otherType");
						}
						if(text) this._insertNode(this.doc, topClassNode, text, 'AddDate', val);
					}
				}
			}
		},
		
		_updateCustomListNode : function() {
			var topClassNode = DomHelper.getChildNode(this.root);
			// Loop through first class node
			for(;topClassNode; topClassNode=DomHelper.getNextNode(topClassNode)) {
				var parentKey = topClassNode.getAttribute('key');
				var parentVal = topClassNode.getAttribute('value');
				if(parentKey == 'ListId') {	// Apend custom lists
					// If it already has children, romove them
					if(DomHelper.getChildNode(topClassNode) != null) {
						DomHelper.removeChildren(topClassNode);
					}
					var customLists = this.mediaDb.getCustomLists();
					for each (var obj in customLists) {
						var text = obj['ListName'];
						var listId = obj['ListId'];
						if(text && listId) {
							this._insertNode(this.doc, topClassNode, text, 'ListId', listId);
						}
					}
				}
			}
		},
		
		_insertNode : function(doc, node, text, key, value) {
			var childNode = doc.createElement("category");
			childNode = node.appendChild(childNode);
			if(text) childNode.setAttribute("text", text);
			if(key) childNode.setAttribute("key", key);
			if(value != null) childNode.setAttribute("value", value);
		}
	}
}