/* Wrapper for richListBox element
   Created by Zoominla, 02-18-2009         
*/

if (typeof(TREE_BOX_INCLUDE) != "boolean") {
    TREE_BOX_INCLUDE = true;
    
function CTreeBox(aTree) {
    if(!aTree) {
        throw new Error('CTreeBox init failed! Tree element may not exist.');
    }
    this.tree = aTree;
    this.curIndex = -1;
    this.curSQL = null;
    this.sqlParamsArr = [];
    this.navListNames = [];
    this.mediaDb = null;
}

CTreeBox.prototype = {
    initialize : function(mediadb) {
        this.mediaDb = mediadb;
        if(!this.mediaDb) {
            throw new Error("NavCatalog init failed!");
        }
    },
    
    setCurrentSQL : function(sqlStr) {
        if(!sqlStr) return;
        this.curSQL = sqlStr;
    },

    setSQLParamsArr : function(paramArr) {
        if(!(paramArr instanceof Array)) return;
        this.sqlParamsArr = paramArr;
    },

    setNavListNamesArr : function(paramArr) {
        if(!(paramArr instanceof Array)) return;
        this.navListNames = paramArr;
    },
    
    update : function(updateMsg) {
        var recordsArr = null;
        if(updateMsg == NavCatalog.LISTENER_FLAGS_ONCONFIRM || updateMsg == NavCatalog.LISTENER_FLAGS_ONRETURN) {
            // Update navigate list box
            if(this.navListNames.length > 0) {
                recordsArr = this.navListNames;
            }
        } else {
            recordsArr = this.execSQL();
        }
        
        if(recordsArr != null) {
            var curTreeView = new ClassTreeView();
            for each (var recordObj in recordsArr) {
                curTreeView.addTopObject(recordObj);
            }
            this.tree.view = curTreeView;
        }
    },
    
    setCurrentIndex : function() {
        this.curIndex = this.tree.currentIndex;
    },
    
    getCurrentIndex : function() {
        return this.curIndex;
    },
    
    getSelectedIndex : function() {
        return this.tree.currentIndex;
    },
    
    getLength: function() {
        return this.tree.rows;  
    },
    
    execSQL : function() {
        if(!this.curSQL || !this.mediaDb) return null;
        // The paramter count usally less than 4
        switch(this.sqlParamsArr.length) {
            case 0:
                return this.mediaDb.queryBySQL(this.curSQL);
            case 1:
                return this.mediaDb.queryBySQL(this.curSQL, this.sqlParamsArr[0]);
            case 2:
                return this.mediaDb.queryBySQL(this.curSQL, this.sqlParamsArr[0], this.sqlParamsArr[1]);
            case 3:
                return this.mediaDb.queryBySQL(this.curSQL, this.sqlParamsArr[0], this.sqlParamsArr[1], this.sqlParamsArr[2]);
            case 4:
                return this.mediaDb.queryBySQL(this.curSQL, this.sqlParamsArr[0], this.sqlParamsArr[1], this.sqlParamsArr[2], this.sqlParamsArr[3]);
            case 5:
                return this.mediaDb.queryBySQL(this.curSQL, this.sqlParamsArr[0], this.sqlParamsArr[1], this.sqlParamsArr[2], this.sqlParamsArr[3], this.sqlParamsArr[4]);
        }
        return null;
    }
}

}